import os
import shutil

import docker
from docker.errors import APIError, NotFound

AWS_REGION = "ap-southeast-2"
ECR_REGISTRY = "public.ecr.aws/z3i0q5x8"
GAME_SERVER_ECR_REPO = "cq-game-server"

DOCKER_CLIENT = docker.from_env()


def get_server_image_tag():
    return f"{ECR_REGISTRY}/{GAME_SERVER_ECR_REPO}:latest"


def get_client_image_tag():
    return "cq-local-dev-client:latest"


def pull_latest_game_server():
    print("Pulling the game server...", end=" ", flush=True)
    game_server_image = get_server_image_tag()
    try:
        DOCKER_CLIENT.images.pull(game_server_image)
        print("Done!")
    except APIError:
        print("Failed to pull the game server!")
        return False
    return True


def check_dockerfile_exists():
    current_dir = os.getcwd()
    dockerfile_path = os.path.join(current_dir, "Dockerfile")

    if os.path.isfile(dockerfile_path):
        print("Dockerfile found.")
    else:
        raise Exception(
            "Dockerfile not found! Make sure you run `cq23_run` from the directory that contains your bot's"
            "Dockerfile."
        )


def build_and_tag_image(image_tag):
    print("Building the image...")
    current_dir = os.getcwd()
    dockerfile_path = os.path.join(current_dir, "Dockerfile")

    # Build the Docker image
    image, _ = DOCKER_CLIENT.images.build(
        path=current_dir, dockerfile=dockerfile_path, tag=image_tag, rm=True
    )

    # Print the ID of the built image
    print("Docker image built with ID:", image.id)


def copy_replay_files(game_files_abs_path):
    volume_name = "cq-game-replay"
    local_dir = "replay_files"

    # Ensure the local directory exists and is empty
    os.makedirs(local_dir, exist_ok=True)
    for root, dirs, files in os.walk(local_dir):
        # Delete all files in the current directory
        for file in files:
            os.remove(os.path.join(root, file))
        # Delete all subfolders and their contents
        for dir in dirs:
            shutil.rmtree(os.path.join(root, dir))

    # Get the Docker volume and its associated container
    try:
        DOCKER_CLIENT.volumes.get(volume_name)
    except NotFound:
        print("Volume does not exist! Something has failed.")
        return False

    cmd = ["/bin/sh", "-c", f"cp -r /data/* /{local_dir}"]
    replay_files_dir = os.path.join(game_files_abs_path, local_dir)
    DOCKER_CLIENT.containers.run(
        "busybox",
        cmd,
        detach=True,
        remove=True,
        volumes={
            volume_name: {"bind": "/data", "mode": "ro"},
            replay_files_dir: {"bind": f"/{local_dir}", "mode": "rw"},
        },
    )

    return True



import shutil

import docker
from docker.errors import NotFound


def cleanup(*args):
    # Create a Docker client
    client = docker.from_env()

    # Get all containers
    containers = client.containers.list(all=True)

    # Delete containers with names starting with 'cq-'
    for container in containers:
        if container.name.startswith("cq_"):
            print("> Deleting container", container.name)
            container.remove(force=True)

    # Get all images
    images = client.images.list(all=True)

    # Delete images with names starting with 'cq_'
    for image in images:
        for tag in image.tags:
            if tag.startswith("cq_"):
                print("> Deleting image", tag)
                client.images.remove(image.id, force=True)
                break

    # Get the volume
    try:
        volume = client.volumes.get("cq-game-replay")

        # Delete the volume
        print("> Deleting volume cq-game-replay")
        volume.remove()
    except NotFound:
        pass

    print("All CodeQuest containers, images and volumes deleted.")

    try:
        print("Deleting game files...", end=" ", flush=True)
        shutil.rmtree(".game_files")
        print("deleted.")
    except FileNotFoundError:
        print("already deleted.")



import json
import os
import subprocess
import sys

from . import docker_tools

MATCH_TIMEOUT_SECONDS = 12 * 60  # This should ideally match the one in game worker


def run_gcs(gcs_folder_name, game_map=None):
    gcs_src_dir = os.path.join(gcs_folder_name, "src")
    clients_file_content = [
        {
            "id": "1",
            "name": "Your Code - 1",
            "image": docker_tools.get_client_image_tag(),
        },
        {
            "id": "2",
            "name": "Your Code - 2",
            "image": docker_tools.get_client_image_tag(),
        },
    ]
    clients_file_address = "clients.json"

    with open(os.path.join(gcs_src_dir, clients_file_address), "w") as f:
        f.write(json.dumps(clients_file_content))

    subprocess_args = [
        sys.executable,
        "controller.py",
        docker_tools.get_server_image_tag(),
        clients_file_address,
    ]

    if game_map:
        subprocess_args.append("--server-arg")
        subprocess_args.append("-m " + str(game_map))

    print("Starting the game...")
    subprocess.run(subprocess_args, timeout=MATCH_TIMEOUT_SECONDS, cwd=gcs_src_dir)
    print("Game finished.")

    os.remove(os.path.join(gcs_src_dir, clients_file_address))



import sys

from cleanup.command import cleanup
from run_game.command import run_game

from .utils import restore_cwd


def help_message():
    print("Help me")


@restore_cwd
def route_command():
    command_args = sys.argv[1:]

    if not command_args:
        help_message()
    elif command_args[0].lower() == "run":
        run_game(*command_args[1:])
    elif command_args[0].lower() == "cleanup":
        cleanup(*command_args[1:])



import os
from functools import wraps


def restore_cwd(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        original_cwd = os.getcwd()
        try:
            return func(*args, **kwargs)
        finally:
            os.chdir(original_cwd)

    return wrapper
